/******************************************************
 *
 * Name:         exploring-alter-table-syntax1.sql
 *     
 * Design Phase:
 *     Author:   John Miner
 *     Date:     05-01-2022
 *     Purpose:  Test out the alter table syntax.
 *
 ******************************************************/


-- https://docs.microsoft.com/en-us/sql/t-sql/statements/alter-table-transact-sql?view=sql-server-ver15

-- Switch databases
USE [AdventureWorksLT2012]
GO



--
-- 1 - Files used by database
--

-- Show file size
SELECT 
  F.file_id,
  F.type,
  F.type_desc,
  F.name,
  F.physical_name, 
  F.size * 8/1024 'Size (MB)',
  F.max_size
FROM 
  sys.database_files as F
  GO



--
-- 2 - Tables inside database
--

-- Show tables
SELECT 
  S.name AS SCHEMA_NM1,
  S.schema_id AS SCHEMA_ID1,
  T.name AS TABLE_NM1,  
  T.object_id AS TABLE_ID1,
  T.type AS TYPE_CD1,
  T.type_desc AS TYPE_NM1,
  OBJECTPROPERTYEX(t.object_id, 'Cardinality') as ROW_CNT1,
  T.modify_date AS MODIFY_DTE1
FROM 
  sys.tables AS T
JOIN
  sys.schemas AS S
ON 
  T.schema_id = S.schema_id
GO



--
-- 3 - Show table details (describ, contents, space)
--
  
-- Get table definition
sp_help 'SalesLT.Product'

-- Show products
SELECT * FROM SalesLT.Product
GO

-- Space of table
sp_spaceused 'SalesLT.Product'



--
-- 4 - How to drop a column
--

-- Remove columns
ALTER TABLE SalesLT.Product 
DROP COLUMN ThumbNailPhoto, ThumbnailPhotoFileName;

-- Fails showing missing columns
SELECT ThumbNailPhoto, ThumbnailPhotoFileName FROM SalesLT.Product
GO

-- Rebuild the table
ALTER TABLE SalesLT.Product REBUILD
GO

-- Space after removing columns
sp_spaceused 'SalesLT.Product'
GO

-- Show products
SELECT TOP 5 * FROM SalesLT.Product
GO


--
-- 5 - How to rename a column
--

-- Drop the constraint
ALTER TABLE SalesLT.Product DROP CONSTRAINT [CK_Product_Weight]
GO

-- Rename column
EXEC sp_rename 'SalesLT.Product.Weight', 'WeightGrams', 'COLUMN';  
GO  

-- Create new constraint
ALTER TABLE [SalesLT].[Product] WITH NOCHECK ADD CONSTRAINT [CK_Product_Weight] CHECK (([WeightGrams]>(0.00)))
GO

-- Enable contraint
ALTER TABLE [SalesLT].[Product] CHECK CONSTRAINT [CK_Product_Weight]
GO

-- Show products
SELECT TOP 5 * FROM SalesLT.Product
GO


--
-- 6 - How to add a column
--

-- Add new field
ALTER TABLE [SalesLT].[Product] ADD [WeightPounds] [decimal](8, 2) NULL;
GO

-- Set value
UPDATE [SalesLT].[Product] SET [WeightPounds] = [WeightGrams] / 453.59237;
GO

-- Show the table
SELECT ProductId, WeightGrams, WeightPounds FROM [SalesLT].[Product] WHERE WeightGrams IS NOT NULL
GO



--
-- 7 - How to alter a column
--

-- Grams - Change from (8,2) -> (10,4) 
ALTER TABLE SalesLT.Product ALTER COLUMN [WeightGrams] decimal (10,4); 
GO

-- Pounds - Change from (8,2) -> (10,4) 
ALTER TABLE SalesLT.Product ALTER COLUMN [WeightPounds] decimal (10,4); 
GO


-- Show the table
SELECT ProductId, WeightGrams, WeightPounds FROM [SalesLT].[Product] WHERE WeightGrams IS NOT NULL
GO



--
-- 8 - Calculated column
--

-- Add new field
ALTER TABLE [SalesLT].[Product] DROP COLUMN [WeightPounds];
GO

-- Add new field
ALTER TABLE [SalesLT].[Product] ADD [WeightPounds] AS CAST(([WeightGrams] / 453.59237) AS decimal (10,4));
GO

-- Show the table
SELECT ProductId, WeightGrams, WeightPounds FROM [SalesLT].[Product] WHERE WeightGrams IS NOT NULL
GO


-- SELECT * INTO bac.Product FROM SalesLt.Product



--
-- 9 - Use view to enforce order
--

CREATE VIEW vProductsWeights
AS
SELECT 
    [ProductID]
   ,[Name]
   ,[ProductNumber]
   ,[Color]
   ,[StandardCost]
   ,[ListPrice]
   ,[Size]
   ,[WeightGrams]
   ,[WeightPounds]
   ,[ProductCategoryID]
   ,[ProductModelID]
   ,[SellStartDate]
   ,[SellEndDate]
   ,[DiscontinuedDate]
   ,[rowguid]
   ,[ModifiedDate]
FROM 
    [AdventureWorksLT2012].[SalesLT].[Product]
GO