/******************************************************
 *
 * Name:         exploring-alter-table-syntax1.sql
 *     
 * Design Phase:
 *     Author:   John Miner
 *     Date:     05-01-2022
 *     Purpose:  Test out the alter table syntax.
 *
 *****************************************************/


--
--  1 - Create and fill new table
--
 
-- Use correct database
USE [AdventureWorksLT2012]
GO

-- Drop table
DROP TABLE IF EXISTS [tmp].[Product];
GO

-- Drop old schema
DROP SCHEMA [tmp];
GO

-- Create new schema
CREATE SCHEMA [tmp];
GO

-- Create table
CREATE TABLE [tmp].[Product]
(
	[ProductID] [int] IDENTITY(1,1) NOT NULL,
	[Name] [dbo].[Name] NOT NULL,
	[ProductNumber] [nvarchar](25) NOT NULL,
	[Color] [nvarchar](15) NULL,
	[StandardCost] [money] NOT NULL,
	[ListPrice] [money] NOT NULL,
	[Size] [nvarchar](5) NULL,
	[WeightGrams] [decimal](10, 4) NULL,
	[WeightPounds]  AS (CONVERT([decimal](10,4),[WeightGrams]/(453.59237),0)),
	[ProductCategoryID] [int] NULL,
	[ProductModelID] [int] NULL,
	[SellStartDate] [datetime] NOT NULL,
	[SellEndDate] [datetime] NULL,
	[DiscontinuedDate] [datetime] NULL,
	[rowguid] [uniqueidentifier] ROWGUIDCOL  NOT NULL,
	[ModifiedDate] [datetime] NOT NULL,

    CONSTRAINT [PK_Product_ProductID2] PRIMARY KEY CLUSTERED 
    (
	[ProductID] ASC
    ),

    CONSTRAINT [AK_Product_Name2] UNIQUE NONCLUSTERED 
    (
	[Name] ASC
    ),

    CONSTRAINT [AK_Product_ProductNumber2] UNIQUE NONCLUSTERED 
    (
	[ProductNumber] ASC
    ),

    CONSTRAINT [AK_Product_rowguid2] UNIQUE NONCLUSTERED 
    (
	[rowguid] ASC
    )
) ON [PRIMARY] 
GO

ALTER TABLE [tmp].[Product] ADD  CONSTRAINT [DF_Product_rowguid2]  DEFAULT (newid()) FOR [rowguid]
GO

ALTER TABLE [tmp].[Product] ADD  CONSTRAINT [DF_Product_ModifiedDate2]  DEFAULT (getdate()) FOR [ModifiedDate]
GO

ALTER TABLE [tmp].[Product]  WITH CHECK ADD  CONSTRAINT [FK_Product_ProductCategory_ProductCategoryID2] FOREIGN KEY([ProductCategoryID])
REFERENCES [SalesLT].[ProductCategory] ([ProductCategoryID])
GO

ALTER TABLE [tmp].[Product] CHECK CONSTRAINT [FK_Product_ProductCategory_ProductCategoryID2]
GO

ALTER TABLE [tmp].[Product]  WITH CHECK ADD  CONSTRAINT [FK_Product_ProductModel_ProductModelID2] FOREIGN KEY([ProductModelID])
REFERENCES [SalesLT].[ProductModel] ([ProductModelID])
GO

ALTER TABLE [tmp].[Product] CHECK CONSTRAINT [FK_Product_ProductModel_ProductModelID2]
GO

ALTER TABLE [tmp].[Product]  WITH CHECK ADD  CONSTRAINT [CK_Product_ListPrice2] CHECK  (([ListPrice]>=(0.00)))
GO

ALTER TABLE [tmp].[Product] CHECK CONSTRAINT [CK_Product_ListPrice2]
GO

ALTER TABLE [tmp].[Product]  WITH CHECK ADD  CONSTRAINT [CK_Product_SellEndDate2] CHECK  (([SellEndDate]>=[SellStartDate] OR [SellEndDate] IS NULL))
GO

ALTER TABLE [tmp].[Product] CHECK CONSTRAINT [CK_Product_SellEndDate2]
GO

ALTER TABLE [tmp].[Product]  WITH CHECK ADD  CONSTRAINT [CK_Product_StandardCost2] CHECK  (([StandardCost]>=(0.00)))
GO

ALTER TABLE [tmp].[Product] CHECK CONSTRAINT [CK_Product_StandardCost2]
GO

ALTER TABLE [tmp].[Product]  WITH NOCHECK ADD  CONSTRAINT [CK_Product_Weight2] CHECK  (([WeightGrams]>(0.00)))
GO

ALTER TABLE [tmp].[Product] CHECK CONSTRAINT [CK_Product_Weight2]
GO



--
-- 2 - Use three schemas (tmp, bac, saleslt)
--

select s.name, t.* 
from sys.tables t
join sys.schemas s 
on t.schema_id = s.schema_id
where t.name = 'Product'
go


--
--  3 - Insert re-organzied data into [tmp]
--

-- Allow inserts of ids
SET IDENTITY_INSERT [tmp].[Product] ON
GO

-- Insert data
INSERT INTO [tmp].[Product]
(
    [ProductID]
   ,[Name]
   ,[ProductNumber]
   ,[Color]
   ,[StandardCost]
   ,[ListPrice]
   ,[Size]
   ,[WeightGrams]
   ,[ProductCategoryID]
   ,[ProductModelID]
   ,[SellStartDate]
   ,[SellEndDate]
   ,[DiscontinuedDate]
   ,[rowguid]
   ,[ModifiedDate]
)
SELECT 
    [ProductID]
   ,[Name]
   ,[ProductNumber]
   ,[Color]
   ,[StandardCost]
   ,[ListPrice]
   ,[Size]
   ,[WeightGrams]
   ,[ProductCategoryID]
   ,[ProductModelID]
   ,[SellStartDate]
   ,[SellEndDate]
   ,[DiscontinuedDate]
   ,[rowguid]
   ,[ModifiedDate]
FROM 
    [AdventureWorksLT2012].[SalesLT].[Product]
GO

-- Dont allow inserts of ids
SET IDENTITY_INSERT [tmp].[Product] OFF
GO


--
-- Show re-organized data
-- 

SELECT * FROM [tmp].[Product]
GO


--
-- 4 - Shell game - move old and new tables around
-- 

-- Remove schema binding from view
ALTER VIEW [SalesLT].[vProductAndDescription] 
AS 
SELECT 
    p.[ProductID] 
    ,p.[Name] 
    ,pm.[Name] AS [ProductModel] 
    ,pmx.[Culture] 
    ,pd.[Description] 
FROM [SalesLT].[Product] p 
    INNER JOIN [SalesLT].[ProductModel] pm 
    ON p.[ProductModelID] = pm.[ProductModelID] 
    INNER JOIN [SalesLT].[ProductModelProductDescription] pmx 
    ON pm.[ProductModelID] = pmx.[ProductModelID] 
    INNER JOIN [SalesLT].[ProductDescription] pd 
    ON pmx.[ProductDescriptionID] = pd.[ProductDescriptionID];
GO


-- Remove foreign keys (old table)
ALTER TABLE [SalesLT].[Product] DROP CONSTRAINT [FK_Product_ProductModel_ProductModelID]
GO

ALTER TABLE [SalesLT].[Product] DROP CONSTRAINT [FK_Product_ProductCategory_ProductCategoryID]
GO

ALTER TABLE [SalesLT].[SalesOrderDetail] DROP CONSTRAINT [FK_SalesOrderDetail_Product_ProductID] 
GO

-- Tranfer old to bac
ALTER SCHEMA [bac] TRANSFER [SalesLT].[Product];
GO

-- Transfer new to saleslt
ALTER SCHEMA [SalesLT] TRANSFER [tmp].[Product];
GO

-- Reapply foreign keys
ALTER TABLE [SalesLT].[SalesOrderDetail]  WITH CHECK ADD  CONSTRAINT [FK_SalesOrderDetail_Product_ProductID] FOREIGN KEY([ProductID])
REFERENCES [SalesLT].[Product] ([ProductID])
GO

ALTER TABLE [SalesLT].[SalesOrderDetail] CHECK CONSTRAINT [FK_SalesOrderDetail_Product_ProductID]
GO
